@@ -4,9 +4,10 @@ from changepoynt.utils import linalg as lg
 from changepoynt.utils import normalization
 from typing import Callable
 from functools import partial
+from changepoynt.algorithms.base_algorithm import Algorithm
 
 
-class SingularSpectrumTransformation:
+class SingularSpectrumTransformation(Algorithm):
     """
     This class implements all the utility and functionality necessary to compute the SST change point detection
     algorithm as described in:
@@ -45,7 +46,7 @@ class SingularSpectrumTransformation:
     """
 
     def __init__(self, window_length: int, n_windows: int = None, lag: int = None, rank: int = 5, scale: bool = True,
-                 method: str = 'ika', lanczos_rank: int = None, random_rank: int = 10,
+                 method: str = 'ika', lanczos_rank: int = None, random_rank: int = None,
                  feedback_noise_level: float = 1e-3) -> None:
         """
         Initializing the Singular Spectrum Transformation (SST) requires setting a lot of parameters. See the parameters
@@ -84,7 +85,7 @@ class SingularSpectrumTransformation:
 
         :param random_rank: In order to use the randomized singular value decomposition, one needs to provide a
         randomized rank (size of second matrix dimension for the randomized matrix) as specified in [3]. The lower
-        this value, the faster the computation but the higher the error (as the approxiamtion gets worse).
+        this value, the faster the computation but the higher the error (as the approximation gets worse).
 
         :param feedback_noise_level: This specifies the amplitude of additive white gaussian noise added to the dominant
         "eigensignal" of the future behavior when shifting forward. This idea is noted in [2] and initializes
@@ -116,7 +117,7 @@ class SingularSpectrumTransformation:
         if self.random_rank is None:
             # compute the rank as specified in [3] and
             # https://scikit-learn.org/stable/modules/generated/sklearn.utils.extmath.randomized_svd.html
-            self.random_rank = self.rank + 10
+            self.random_rank = min(self.rank + 10, self.window_length, self.n_windows)
 
         # specify the methods and their corresponding functions as lambda functions expecting only the
         # 1) future hankel matrix,
@@ -127,13 +128,13 @@ class SingularSpectrumTransformation:
                                        rank=self.rank,
                                        lanczos_rank=self.lanczos_rank),
                         'svd': partial(_rayleigh_singular_value_decomposition,
-                                         rank=self.rank),
+                                       rank=self.rank),
                         'rsvd': partial(_randomized_singular_value_decomposition,
                                         rank=self.rank,
-                                        randomized_rank=self.lanczos_rank),
+                                        randomized_rank=self.random_rank),
                         'fbrsvd': partial(_facebook_random_singular_value_decomposition,
                                           rank=self.rank,
-                                          randomized_rank=self.lanczos_rank)
+                                          randomized_rank=self.random_rank)
                         }
 
         # check whether the method is correct
@@ -190,7 +191,7 @@ def _transform(time_series: np.ndarray, start_idx: int, window_length: int, n_wi
 
     # create initial vector for ika method with feedback dominant eigenvector as proposed in [2]
     # with a norm of one
-    x0 = np.random.rand(window_length)
+    x0 = np.random.rand(n_windows)
     x0 /= np.linalg.norm(x0)
 
     # initialize a scoring array with no values yet
@@ -200,10 +201,10 @@ def _transform(time_series: np.ndarray, start_idx: int, window_length: int, n_wi
     for idx in range(start_idx, time_series.shape[0]):
 
         # compile the past hankel matrix (H1)
-        hankel_past = _compile_hankel(time_series, idx-lag, window_length, n_windows)
+        hankel_past = lg.compile_hankel(time_series, idx-lag, window_length, n_windows)
 
         # compile the future hankel matrix (H2)
-        hankel_future = _compile_hankel(time_series, idx, window_length, n_windows)
+        hankel_future = lg.compile_hankel(time_series, idx, window_length, n_windows)
 
         # compute the score and save the returned feedback vector
         score[idx], x1 = scoring_function(hankel_past, hankel_future, x0)
@@ -215,32 +216,6 @@ def _transform(time_series: np.ndarray, start_idx: int, window_length: int, n_wi
     return score
 
 
-@jit(nopython=True)
-def _compile_hankel(time_series: np.ndarray, end_index: int, window_size: int, rank: int) -> np.ndarray:
-    """
-    This function constructs a hankel matrix from a 1D time series. Please make sure constructing the matrix with
-    the given parameters (end index, window size, etc.) is possible, as this function does no checks due to
-    performance reasons.
-
-    :param time_series: 1D array with float values as the time series
-    :param end_index: the index (point in time) where the time series starts
-    :param window_size: the size of the windows cut from the time series
-    :param rank: the amount of time series in the matrix
-    :return: The hankel matrix with lag one
-    """
-
-    # make an empty matrix to place the values
-    #
-    # almost no faster way:
-    # https://stackoverflow.com/questions/71410927/vectorized-way-to-construct-a-block-hankel-matrix-in-numpy-or-scipy
-    hankel = np.empty((window_size, rank))
-
-    # go through the time series and make the hankel matrix
-    for cx in range(rank):
-        hankel[:, cx] = time_series[(end_index-window_size-cx):(end_index-cx)]
-    return hankel
-
-
 def _implicit_krylov_approximation(hankel_past: np.ndarray, hankel_future: np.ndarray, x0: np.ndarray,
                                    rank: int, lanczos_rank: int) -> (float, np.ndarray):
     """
@@ -422,7 +397,7 @@ def _facebook_random_singular_value_decomposition(hankel_past: np.ndarray, hanke
 
 
 if __name__ == '__main__':
-
+    from time import time
     # make synthetic step function
     np.random.seed(123)
     length = 300
@@ -438,7 +413,9 @@ if __name__ == '__main__':
     fbrsvd_sst = SingularSpectrumTransformation(31, method='fbrsvd')
 
     # make the scoring
+    start = time()
     ika_sst.transform(x)
+    print((time() - start) / (length * 3))
     svd_sst.transform(x)
     rsvd_sst.transform(x)
     fbrsvd_sst.transform(x)
